import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.util.Properties;
import java.net.URL;
import java.net.URLConnection;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * Read the SOAP envelope file passed as the second
 * parameter, pass it to the SOAP endpoint passed as the first parameter, and
 * save the SOAP envelope passed as a response in a file.
 *
 * This class sends a given SOAP with attachments file to a given URL. The
 * boundary string needs to be given. The value of SOAPAction attribute can
 * be given. The file is read in default charset.
 * The basic authentication required for WPG is set.
 *
 */
public class GenericSOAPClient2
{
	private static String CLASSNAME = "GenericSOAPClient2";
	private static Logger lgr = Logger.getLogger(CLASSNAME);
	//DEFAULT_BOUNDARY = ----=_Part_0_1176548981.1162110985264;

	private BufferedInputStream bin; //for reading response
	private FileInputStream fin;// for reading file
	private OutputStream out;//for writing to connection
	private BufferedOutputStream bos;//for writing the response to a file
	private String SOAPUrl;
	private String file2Send;
	private String SOAPAction = "";
	private String boundary;
	private String userID = "";
	private String user = "" ;
 	private String pwd = "" ;

	public static void main(String[] args)
	{
		GenericSOAPClient2 client = new GenericSOAPClient2(args);
		client.send();
	}

	public GenericSOAPClient2(String[] args)
	{
		lgr.setLevel(Level.ALL);
		lgr.entering(CLASSNAME, "GenericSOAPClient2", args);
		Properties props = null ;

		try
		{
			SOAPUrl = args[0];
			file2Send = args[1];
			boundary = "----=_Part_0_729729216.1171462831125" ;
			userID = args[2] ;
			user = args[3] ;
			pwd = args[4] ;
		}
		catch( Exception ex )
		{
			System.err.println("Usage: java GenericSOAPClient2 <url> <file to send > <userid> <user> <pwd>");
			System.err.println();
			System.err.println();
			ex.printStackTrace() ;
			System.exit(1);
		}



		lgr.exiting(CLASSNAME, "GenericSOAPClient2");
	}

	public void send()
	{
		lgr.entering(CLASSNAME, "send");
		try
		{
			// Create the connection where we're going to send the file.
			URL url = new URL(SOAPUrl);
			URLConnection connection = url.openConnection();
			HttpURLConnection httpConn = (HttpURLConnection) connection;
			lgr.info("Connection created " + httpConn.toString());

			// Open the input file.
			fin = new FileInputStream(file2Send);

//			ByteArrayOutputStream bout = new ByteArrayOutputStream();

			// Copy the SOAP file to the open connection.
			//copy(fin,bout);
			//fin.close();

			//byte[] b = bout.toByteArray();

			// Set the appropriate HTTP parameters.
	/*		httpConn.setRequestProperty( "Content-Length",
										 String.valueOf( b.length ) );
	*/

			File file = new File(file2Send);
			httpConn.setRequestProperty("Content-Length",
			        String.valueOf(file.length()));
			lgr.finest("File length: " + file.length());

			httpConn.setRequestProperty("Content-Type",
			  "multipart/related; type=\"text/xml\"; boundary=\""
			        + boundary + "\"");
			  /*
			  			httpConn.setRequestProperty("Content-Type",
			  			  "text/xml" );
			  			 */

		  	httpConn.setRequestProperty("SOAPAction",SOAPAction);
			httpConn.setRequestMethod("POST");
			httpConn.setDoOutput(true);
			httpConn.setDoInput(true);

			String toEncode = userID +"/" + user +":" + pwd ;
			httpConn.setRequestProperty("Authorization", "Basic " +
			(new com.ibm.misc.BASE64Encoder().encode(toEncode.getBytes())));

			// Everything's set up; send the file.
			OutputStream out = httpConn.getOutputStream();

			int len = 8096;
			byte[] buf = new byte[len];
			int numRead = 0;
			while((numRead = fin.read(buf,0,len)) != -1)
			{
				out.write(buf,0,numRead);
			}
			out.flush();
			out.close();
			lgr.info("Request written.");

			// Read the response and write it to a file.
			lgr.info("Reading response...");
			InputStream inputStream = httpConn.getInputStream();
			bin = new BufferedInputStream (inputStream);

			byte[] input = new byte[len];
			numRead = 0;
			String userDir = System.getProperty("user.dir");
			File tmpFile = File.createTempFile("swa_resp_",".txt",
			        new File(userDir));
			BufferedOutputStream bos = new BufferedOutputStream(
			        new FileOutputStream(tmpFile));
			if(bin.available() != 0)
			{
				while ((numRead = bin.read(input,0,len)) != -1)
				{
					bos.write(input,0,numRead);
				}
				lgr.info("Response written to file "
				        + tmpFile.getAbsolutePath());
			}
			else
			{
				lgr.info("No response received.");
			}
			bos.flush();
			bos.close();
			bin.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			if(out != null)
			{
				try
				{
					out.close();
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
			if(fin != null)
			{
				try
				{
					fin.close();
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
			if(bin != null)
			{
				try
				{
					bin.close();
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
			if(bos != null)
			{
				try
				{
					bos.close();
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
		}
		lgr.exiting(CLASSNAME, "send");
	}

}
